@@ -18,16 +18,27 @@ function (tcl.list)
 }
 
 "showData" <-
-    function (dataframe, colname.bgcolor = "royalblue", rowname.bgcolor = "grey90", 
-              body.bgcolor = "white", colname.textcolor = "white", rowname.textcolor = "darkred", 
-              body.textcolor = "black", font = "Courier 12", maxheight = 30, 
-              maxwidth = 80, title = NULL, rowname.bar = "left", colname.bar = "top", 
-              rownumbers = FALSE, placement = "-20-40") 
+    function (dataframe,
+              colname.bgcolor = "royalblue",
+              rowname.bgcolor = "grey90", 
+              body.bgcolor = "white",
+              colname.textcolor = "white",
+              rowname.textcolor = "darkred", 
+              body.textcolor = "black",
+              font = "Courier 12",
+              maxheight = 30, 
+              maxwidth = 80,
+              title = NULL,
+              rowname.bar = "left",
+              colname.bar = "top", 
+              rownumbers = FALSE,
+              placement = "-20-40") 
 {
     object.name <- deparse(substitute(dataframe))
     if (!is.data.frame(dataframe)) 
         stop(paste(object.name, "is not a data frame"))
-    if (is.numeric(rownumbers) && length(rownumbers) != nrow(dataframe)) 
+    if (is.numeric(rownumbers) &&
+        length(rownumbers) != nrow(dataframe)) 
         stop("rownumbers argument must be TRUE, FALSE or have length nrow(dataframe)")
     require(tcltk) || stop("Tcl/Tk support is absent")
     oldwidth <- options()$width
@@ -56,17 +67,41 @@ function (tcl.list)
     rm(zz, envir = .GlobalEnv)
     datawidth <- max(nchar(yy))
     winwidth <- min(1 + datawidth, maxwidth)
-    hdr <- tktext(base, bg = colname.bgcolor, fg = colname.textcolor, 
-                  font = font, height = 1, width = winwidth)
-    ftr <- tktext(base, bg = colname.bgcolor, fg = colname.textcolor, 
-                  font = font, height = 1, width = winwidth)
-    txt <- tktext(base, bg = body.bgcolor, fg = body.textcolor, 
-                  font = font, height = min(maxheight, nrows), width = winwidth)
-    lnames <- tktext(base, bg = rowname.bgcolor, fg = rowname.textcolor, 
-                     font = font, height = min(maxheight, nrows), width = namewidth)
-    rnames <- tktext(base, bg = rowname.bgcolor, fg = rowname.textcolor, 
-                     font = font, height = min(maxheight, nrows), width = namewidth)
-    xscroll <- tkscrollbar(base, orient = "horizontal", repeatinterval = 1, 
+    hdr <- tktext(base,
+                  bg = colname.bgcolor,
+                  fg = colname.textcolor, 
+                  font = font,
+                  height = 1,
+                  width = winwidth)
+    ftr <- tktext(base,
+                  bg = colname.bgcolor,
+                  fg = colname.textcolor, 
+                  font = font,
+                  height = 1,
+                  width = winwidth)
+    textheight <- min(maxheight, nrows)
+    txt <- tktext(base,
+                  bg = body.bgcolor,
+                  fg = body.textcolor, 
+                  font = font,
+                  height = textheight,
+                  width = winwidth,
+                  setgrid = 1)
+    lnames <- tktext(base,
+                     bg = rowname.bgcolor,
+                     fg = rowname.textcolor, 
+                     font = font,
+                     height = textheight,
+                     width = namewidth)
+    rnames <- tktext(base,
+                     bg = rowname.bgcolor,
+                     fg = rowname.textcolor, 
+                     font = font,
+                     height = textheight,
+                     width = namewidth)
+    xscroll <- tkscrollbar(base,
+                           orient = "horizontal",
+                           repeatinterval = 1, 
                            command = function(...) {
                                tkxview(txt, ...)
                                tkxview(hdr, ...)
@@ -94,7 +129,9 @@ function (tcl.list)
         tkxview.moveto(hdr, xy[1])
         tkxview.moveto(txt, xy[1])
     })
-    yscroll <- tkscrollbar(base, orient = "vertical", repeatinterval = 1, 
+    yscroll <- tkscrollbar(base,
+                           orient = "vertical",
+                           repeatinterval = 1, 
                            command = function(...) {
                                tkyview(txt, ...)
                                tkyview(lnames, ...)
@@ -128,24 +165,26 @@ function (tcl.list)
     tktag.configure(rnames, "notwrapped", wrap = "none")
     tkinsert(txt, "end", paste(paste(yy[-1], collapse = "\n"), 
                                sep = ""), "notwrapped")
-    tkgrid(txt, row = 1, column = 1)
+    tkgrid(txt, row = 1, column = 1, sticky = "nsew")
     if ("top" %in% colname.bar) {
         tkinsert(hdr, "end", paste(yy[1], sep = ""), "notwrapped")
-        tkgrid(hdr, row = 0, column = 1)
+        tkgrid(hdr, row = 0, column = 1, sticky = "ew")
     }
     if ("bottom" %in% colname.bar) {
         tkinsert(ftr, "end", paste(yy[1], sep = ""), "notwrapped")
-        tkgrid(ftr, row = 2, column = 1)
+        tkgrid(ftr, row = 2, column = 1, sticky = "ew")
     }
     if ("left" %in% rowname.bar) {
-        tkinsert(lnames, "end", paste(rowname.text, collapse = "\n"), 
+        tkinsert(lnames, "end",
+                 paste(rowname.text, collapse = "\n"), 
                  "notwrapped")
-        tkgrid(lnames, row = 1, column = 0)
+        tkgrid(lnames, row = 1, column = 0, sticky = "ns")
     }
     if ("right" %in% rowname.bar) {
-        tkinsert(rnames, "end", paste(rowname.text, collapse = "\n"), 
+        tkinsert(rnames, "end",
+                 paste(rowname.text, collapse = "\n"), 
                  "notwrapped")
-        tkgrid(rnames, row = 1, column = 2)
+        tkgrid(rnames, row = 1, column = 2, sticky = "ns")
     }
     tkconfigure(hdr, state = "disabled")
     tkconfigure(ftr, state = "disabled")
@@ -158,5 +197,9 @@ function (tcl.list)
     if (maxwidth < datawidth) {
         tkgrid(xscroll, row = 3, column = 1, sticky = "ew")
     }
+    tkgrid.rowconfigure(base, 1, weight = 1)
+    tkgrid.columnconfigure(base, 1, weight = 1)
+    tkwm.maxsize(base, 1+datawidth, nrows)
+    tkwm.minsize(base, 1+nchar(names(dataframe)[1]), 1)
     invisible(NULL)
 }
